
import org.apache.commons.jexl3.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.plexus.util.IOUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public interface JvmScript
{
	public static JvmScript create() { return new Jexl3Script(); }

	public void initScript(Properties properties, File scriptFile);
	public void initScript(Properties properties, Reader scriptFile);
	public void initScript(Properties properties, String scriptCode);

	public File getScriptFile();
	public void setScriptFile(File scriptFile);

	public void setIncludePaths(List<String> _ic);
	public List<String> getIncludePaths();

	public Properties getScriptProperties();
	public void setScriptProperties(Properties scriptProperties);

	public Object runScript(Map<String,Object> context, String[] args, boolean multiplicity);

	public static abstract class AbstractJvmScript implements JvmScript
	{
		File scriptFile;
		String scriptCode;
		Properties scriptProperties;
		List<String> includePaths;

		@Override
		public List<String> getIncludePaths() {
			return includePaths;
		}

		@Override
		public void setIncludePaths(List<String> includePaths) {
			this.includePaths = includePaths;
		}

		@Override
		public File getScriptFile()
		{
			return scriptFile;
		}

		@Override
		public void setScriptFile(File scriptFile)
		{
			this.scriptFile = scriptFile;
		}

		@Override
		public Properties getScriptProperties()
		{
			return scriptProperties;
		}

		@Override
		public void setScriptProperties(Properties scriptProperties)
		{
			this.scriptProperties = scriptProperties;
		}

		@Override
		public void initScript(Properties properties, File scriptFile)
		{
			this.scriptProperties = new Properties(properties);
			this.scriptFile = scriptFile;

			Reader rdr = null;
			try
			{
				rdr = new FileReader(scriptFile);
				initScript(properties, rdr);
			}
			catch(Exception xe) {}
			finally
			{
				if(rdr!=null)
				{
					IOUtil.close(rdr);
				}
			}
		}

		@Override
		public void initScript(Properties properties, Reader scriptFile)
		{
			String line = "#";
			StringBuilder sb = new StringBuilder();
			BufferedReader rdr = null;
			try
			{
				rdr = new BufferedReader(scriptFile);

				while(line.startsWith("#"))
				{
					line = rdr.readLine();
				}

				sb.append(line+"\n");

				while((line = rdr.readLine())!=null)
				{
					sb.append(line+"\n");
				}
			}
			catch(Exception xe) {}
			finally
			{
				initScript(properties, sb.toString());
			}
		}

		@Override
		public abstract void initScript(Properties properties, String scriptCode);

		@Override
		public abstract Object runScript(Map<String, Object> context, String[] args, boolean multiplicity);
	}

	public static class Jexl3Script extends AbstractJvmScript
	{
		public static Log log = LogFactory.getLog(com.vts.nms.jdbc.knife.jxl.Jexl3Script.class);

		private JexlEngine engine;
		private JexlScript script;

		@Override
		public void initScript(Properties properties, String scriptCode)
		{
			this.scriptProperties=properties;
			this.scriptCode=scriptCode;

			try
			{
				engine = new JexlBuilder().cache(512).strict(true).silent(false).logger(log).create();
				script = engine.createScript(scriptCode);
			}
			catch (Exception e)
			{
				log.warn("error", e);
			}
		}

		@Override
		public Object runScript(Map<String, Object> context, String[] args, boolean multiplicity)
		{
			JexlEngine l_engine = engine;
			JexlScript l_script = script;

			if(multiplicity)
			{
				try
				{
					l_engine = new JexlBuilder().cache(512).strict(true).silent(false).logger(log).create();
					l_script = l_engine.createScript(scriptCode);
				}
				catch (Exception e)
				{
					log.warn("error", e);
				}
			}

			try
			{
				MapContext l_bind = new MapContext();
				for(Map.Entry<String,Object> e : context.entrySet())
				{
					l_bind.set(e.getKey(), e.getValue());
				}

				if(args!=null)
				{
					l_bind.set("args", args);
				}
				else
				{
					l_bind.set("args", new String[0]);
				}

				Object ret = l_script.execute(l_bind);

				return ret;
			}
			catch(Exception xe)
			{
				log.warn("error in file "+scriptFile, xe);
			}

			return null;
		}

	}
}
